/** Request 网络请求工具 更详细的 api 文档: https://github.com/umijs/umi-request */
import { extend } from 'umi-request';
import { notification } from 'antd';

const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
};
/** 异常处理程序 */

const errorHandler = (error) => {
  const { response } = error;
  if (response && response.status) {
    const errorText = codeMessage[response.status] || response.statusText;
    const { status, url } = response;
    notification.error({
      message: `请求错误 ${status}: ${url}`,
      description: errorText,
    });
  } else if (error.code && error.message) {
    throw new Error(error.message)
  } else {
    notification.error({
      description: '您的网络发生异常，无法连接服务器',
      message: '网络异常',
    });
  }

  return response;
};
/** 配置request请求时的默认参数 */

const request = extend({
  prefix: process.env.NODE_ENV === 'production' ? '/api/admin' : '/api/admin', 
  errorHandler,
  // 默认错误处理
  credentials: 'include', // 默认请求是否带上cookie
});

request.interceptors.request.use((url, options) => {
  const headers = options.headers || {}
  const token = localStorage.getItem('token') ? { 'X-Authorization-JWT': localStorage.getItem('token') } : {}

  return {
    url,
    options: {
      ...options,
      headers: {
        ...headers,
        ...token,
      },
    }
  }
})

request.interceptors.response.use(async (response) => {
  const token = response.headers.get('x-authorization-jwt')
  if (token) {
    localStorage.setItem('token', token)
  }

  const contextType = response.headers.get('content-type')
  if (contextType.indexOf('json') > -1) {
    const result = await response.clone().json()
    if (result.code === 1000) {
      return result.data
    }
    return Promise.reject(result)
  }

  return response
})

export default request;

// 专门为 ProTable 定制的 request
// https://procomponents.ant.design/components/table#request
export const queryTable = (url) => {
  return (params, sort, filter) => {
    return new Promise((resolve, reject) => {
      const { current, pageSize, ...leftParams } = params
      const options = {
        params: {
          pageNum: current,
          pageSize,
          ...leftParams,
        }
      }

      request(url, options).then(res => {
        // 成功返回 success: true
        resolve({
          success: true,
          total: res.total,
          data: res.records
        })
      }).catch((e) => {
        notification.error({
          message: e.message,
        });
        // 失败返回 success: false
        resolve({
          success: false,
          total: 0,
          data: []
        })
      })
    })
  }
}
